/*******************************************************************************
* 
*    ULTRA STEREO LABS CONFIDENTIAL
*    _____________________________________________________________________
* 
*       Copyright (c) 2014 - 2015 Ultra Stereo Labs Incorporated 
*                         All Rights Reserved.
* 
*    NOTICE:  All information contained herein is, and remains
*    the property of Ultra Stereo Labs Incorporated and its suppliers,
*    if any.  The intellectual and technical concepts contained
*    herein are proprietary to Ultra Stereo Labs Incorporated
*    and its suppliers and may be covered by U.S. and Foreign Patents,
*    in process, and are protected by trade secret or copyright law.
*    Dissemination of this information or reproduction of this material
*    is strictly forbidden unless prior written permission is obtained
*    from Ultra Stereo Labs Incorporated. 181 Bonetti Drive, San Luis Obispo, 
*    California 93401-7397, USA – www.uslinc.com
*
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
*
*******************************************************************************/
//---------------------------------------------------------------------------
// ScreenController
//
// This controller request all LSS data items and then updateds bindings 
// via a watch on the collection. The view contains 5 entries by default and
// the watch event updates the number of entries in the table.
//---------------------------------------------------------------------------
app.controller('ScreenController', function ($scope, $timeout, $filter, $rootScope, allDataListSharedFactory, 
            getParametersFactory, allScreensSharedFactory, configurationFactory,  dbRequestService) {
    // Get Global parameters from factories
    $scope.getParameters = getParametersFactory;
    $scope.settings = configurationFactory;
    $scope.screenlist = [];         //Populate with minimal data for all theaters
    // Initialize variables for the view
    $scope.screen = [];
    $scope.screendata = [];
    $scope.tab = 1;                 //Graph or Data tab open
    $scope.opened = [false,false];  //Date-Picker shows
    $scope.graphs = [];
    $scope.checked = [];
    $scope.checkall = false;
    $scope.filteredscreens = [];
    $scope.uniquetheaters=[];
    $scope.datataborder= 'x'; //Order data view by date
    $scope.dtrev = false;
    // Instance specific chart settings - Make changes for the entire scope (screens) here:
    $scope.settings.chartSettings.chart['height']=450;  //Need to force to resize to device
    $scope.settings.chartSettings.title.enable = false;
    $scope.settings.chartSettings.chart['xAxis'] = {
        axisLabel: 'Date',
        tickFormat: function(d){ 
            return d3.time.format($scope.settings.ChartDateFormat)(new Date(d*1000)); 
        }
    };
    $scope.settings.chartSettings.chart['yAxis']= {
        axisLabel: 'Field Value',
        tickFormat: function(d){
            if(d===null){return "None";}
            return (TickToString(d)+" "+$scope.graphdata[0].values[0].units);
        },
        axisLabelDistance: 6
    };
    $scope.graphdata = [   //Initialize graph data array
                {values:[],key:"Description",color:'#33F'},
                {values:[],key:'Maximum',color:'#f50'},
                {values:[],key:'Minimum',color:'#f00'}
            ];

                ////// Controller Functions ///////
    
    // Controllers for the Graph panes
    // Track which descriptions are checked, add delete from the collection
    $scope.myval = "";
    $scope.$watchCollection('checked', function(newNames, oldNames){
        // Remove stale graphs
        var i=0;
        while(i < $scope.graphs.length){
            if(newNames.indexOf($scope.graphs[i].description) < 0){
                console.log("-- remove -- "+$scope.graphs[i].description);
                $scope.graphs.splice(i,1);
            }
            else{ i++; }
        }
        //Add any new graphs
        for (var i in newNames){
            if( newNames[i] !== false && newNames[i] !== null){
                var result = $.grep($scope.graphs, function(e){ return e.description === newNames[i]; });
                if( result.length === 0){
                    $scope.addGraph(newNames[i]);
                }
                console.log("** added **"+newNames[i]);
            }
        }
    });
    // Add a new graph to the array if it is not present
    $scope.addGraph = function(desc){
        // Verify it is not already graphed
        var result = $.grep($scope.graphs, function(e){ return e.description === desc; });
        if (result.length === 0){
            // Generate a new GraphClass with the defaults
            var newgraph = new GraphClass(desc,
                                    [   //Initialize graph data array
                                        {values:[],key:desc,color:'#33F'},
                                        {values:[],key:'Maximum',color:'#f50'},
                                        {values:[],key:'Minimum',color:'#f00'}
                                    ],
                                    $scope.settings.chartSettings);
            // Add labels and tick formats (must be done here so that the $scope references correctly to this controller).
            newgraph.settings.chart['yAxis']= {
                axisLabel: "",
                tickFormat: function(d){   
                    if(d===null){return "None";}
                    if(this.parentNode !== undefined){
                       return (TickToString(d))+" "+this.parentNode.parentNode.__data__[0].values[0].units;
                    }
                    return (TickToString(d));
                }
            };
            //newgraph.settings.chart['forceY']=[0, 100];  // Chart ranging
            // 
            // Discovered that NVd3 only has 1 active settings object; doing this
            // individually affects all shown graphs.  
            // TODO: New Graph class to be written.
            // 
            // TODO: Find a way to reference units for tooltip and first/last tick calls
            
            //  Add and get data for the plot.
            $scope.graphs.push(newgraph);
            $scope.updateGraph(desc);
        }
    };
    // Remove a graph from the array if it is present
    $scope.delGraph = function(desc){
        var i = $scope.checked.indexOf(desc);
        if (i>=0){$scope.checked[i]=null;}
    };
    // Update the dataset for one of the graphs
    $scope.updateGraph = function($desc){    
        var result = $.grep($scope.graphs, function(e){ return e.description === $desc; });
        dbRequestService.dbGraphRequest(result[0].data, EnumDbCmds.GRAPH_DATA, 
            $scope.getParameters.startdate.getTime()/1000, $scope.getParameters.enddate.getTime()/1000, 
            $scope.getParameters.theater, $scope.getParameters.screen, $desc,
            $scope.getParameters.orgkey);
    };
    // For the "CheckAll" button
    $scope.selectAllCB = function(){
        if($scope.checkall){
            var myarr = [];
            // use the filtered descriptions from the input object in the html
            for( var i in $scope.filteredscreens ){
                myarr.push($scope.filteredscreens[i].Description);
            }
            $scope.checked = myarr;
        }
        else{
            $scope.checked.length = 0;
        }
    };    
    // Do a request for all the data
    dbRequestService.dbRequest($scope.screenlist, EnumDbCmds.OVERVIEW, "", "");
    // Attached a watcher on the list object, collection.
    $scope.$watchCollection('screenlist', function(newNames, oldNames){
        if(newNames.length < 1)
            return;

        console.log('ScreenController: Collection update length ='+newNames.length);
        $scope.totalItems = newNames.length;
        $scope.changeScreen();
    });
    // Change operations for the shown data
    $scope.changeTheater = function (){
        $scope.getParameters.theater = $scope.uniquetheaters[$scope.selectedtheater].TheaterNumber;
        $scope.getParameters.orgkey = $scope.uniquetheaters[$scope.selectedtheater].OrgKey;
        $scope.changeScreen();
    };
    //Handler for changes in sleected screen.
    $scope.changeScreen = function (){
        if($scope.getParameters.screen.length>0 && $scope.getParameters.theater.length>0){
            var requestObj = {
                "param1":$scope.getParameters.theater, 
                "param2":$scope.getParameters.screen, 
                "OrgKey":$scope.getParameters.orgkey
            };    
            dbRequestService.dbObjRequest($scope.screen, EnumDbCmds.LSS_DATA, requestObj);
            dbRequestService.dbRequest($scope.screendata, EnumDbCmds.THEATER_OVERVIEW, $scope.getParameters.startdate.getTime()/1000, $scope.getParameters.enddate.getTime()/1000);
            console.log("Descriptions available: "+$scope.screendata.length);
        }
        else {
            $scope.getParameters.description = "";
        }
        $scope.changeGraph();
    };
    // Click handler for LSS data table (show graphs).
    // When a change is made, reaquire the data
    $scope.changeGraph = function (){
        for (var i in $scope.graphs){
            //Get the data to graph into the graphing data collection
            dbRequestService.dbGraphRequest($scope.graphs[i].data, EnumDbCmds.GRAPH_DATA,
                $scope.getParameters.startdate.getTime()/1000, $scope.getParameters.enddate.getTime()/1000, 
                $scope.getParameters.theater, $scope.getParameters.screen, $scope.graphs[i].description,
                $scope.getParameters.orgkey);
            // TODO: Add a way to update multi-graphs  -  New AJAX call?
        }
    };
    $scope.filter = function() {
        $timeout(function() { 
            $scope.filteredItems = $scope.filtered.length;
        }, 10);
//        $scope.parseLatest();
    };
    //sort function for LSS Table headers
    $scope.sort_by = function(predicate) {
        if ($scope.predicate === predicate){ $scope.reverse = !$scope.reverse;}
        else { $scope.reverse = false; }
        $scope.predicate = predicate; 
    };
    // For the DatePicker popup:
    $scope.open = function($event, $index) {
      $event.preventDefault();
      $event.stopPropagation();

      $scope.opened[$index] = true;
    };
    //On Load get the graph
    //System is more stable with this delay
    $scope.loadDesc = function(){
        if($scope.getParameters['description'].length > 0){
            if($scope.getParameters['theater'].length>0 && $scope.getParameters['screen'].length>0){
                $scope.checked.push($scope.getParameters['description']);
            }else{ setTimeout($scope.loadDesc(),1000); }
        }
    };
    $scope.loadPoint = function(){
        if('id' in $scope.getParameters){
            $scope.idrecord = [];                
            dbRequestService.dbRawDataRequest( $scope.idrecord, EnumDbCmds.RawDataRequest, [$scope.getParameters['id']] );
            $scope.$watchCollection('idrecord', function(newNames, oldNames){
                angular.forEach($scope.graphs, function(item){
                    if(item.description === $scope.idrecord[0].Description){
                        item.data.push({
                            values:[{x:parseFloat($scope.idrecord[0].DateTime),
                                    y:parseFloat($scope.idrecord[0].Value),
                                    units:$scope.idrecord[0].Units}],
                            key:"Reference",
                            color:'#ff0'});
                        
                    }
                });
            });
        }
    };
    setTimeout($scope.loadDesc(),1000);
    setTimeout($scope.loadPoint(),2000);
});