/*******************************************************************************
* 
*    ULTRA STEREO LABS CONFIDENTIAL
*    _____________________________________________________________________
* 
*       Copyright (c) 2014 - 2015 Ultra Stereo Labs Incorporated 
*                         All Rights Reserved.
* 
*    NOTICE:  All information contained herein is, and remains
*    the property of Ultra Stereo Labs Incorporated and its suppliers,
*    if any.  The intellectual and technical concepts contained
*    herein are proprietary to Ultra Stereo Labs Incorporated
*    and its suppliers and may be covered by U.S. and Foreign Patents,
*    in process, and are protected by trade secret or copyright law.
*    Dissemination of this information or reproduction of this material
*    is strictly forbidden unless prior written permission is obtained
*    from Ultra Stereo Labs Incorporated. 181 Bonetti Drive, San Luis Obispo, 
*    California 93401-7397, USA – www.uslinc.com
*
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
*
*******************************************************************************/
//---------------------------------------------------------------------------
// addUserUserController
// 
// Similar to the signup except that the user must be a manager or
// system administrator to successfully add a user.
//---------------------------------------------------------------------------
app.controller('addUserController', function ($scope, $rootScope, 
    $location, Data, dbRequestService) {

    //-----------------------------------------------------------------------
    // User Level
    // 
    // Create a list for user access levels, the default is only user.
    //-----------------------------------------------------------------------
    $scope.userLevels = [
        {
            text: 'User',
            value: EnumUserLevels.USER
        }];
     
    //-----------------------------------------------------------------------
    // Notification Levels
    // 
    // Create a list of items for the dropdown.
    //-----------------------------------------------------------------------
    $scope.notifyLevels = [
        {
            text: 'None',
            value: EnumNotifyLevels.NONE
        }, {
            text: 'Errors Only',
            value: EnumNotifyLevels.ERRORSONLY
        }, {
            text: 'All',
            value: EnumNotifyLevels.ALL
        }
    ];
    
    //-----------------------------------------------------------------------
    //                               Data 
    //-----------------------------------------------------------------------
    // public binding properties
    $scope.newUser = new UserStruct()
    $scope.passwordConfirm = "";
    $scope.companyList = [];
        
    $scope.usersList = [];
    $scope.usersDataGridOptions = {
        data:$scope.usersList,
        enableRowSelection: false,
        enableSelectAll: false,
        multiSelect: false,
        minRowsToShow:12,
        enableColumnMenus: false,
        enableHorizontalScrollbar:0,
        enableVerticalScrollbar:1,
        pagingPageSizes:1,
        pagingPageSize: 1,
        columnDefs: [{field: 'Name', displayName: 'User Name', width: '*'},
                     {field: 'Company', displayName: 'Company', width: '25%'}, 
                     {field: 'Email', displayName: 'Email Address', width: '50%'}]      
    };

    var defaultNewUser = angular.copy($scope.newUser);

    //-----------------------------------------------------------------------
    // addUser
    //
    // Sends an add user Web request with the new user's properties.
    //-----------------------------------------------------------------------
    $scope.addUser = function(customer) {
        Data.post('addUser', {
            customer: customer
        }).then(function (results) {
            Data.toast(results);
            if (results.status === "success") {
                $location.path('adduser');
                dbRequestService.dbRequest($scope.usersDataGridOptions.data, EnumDbCmds.USERS);
            }
        });
    };

    //-----------------------------------------------------------------------
    // clearForm
    //
    // Resets the form as viewed when refreshed.
    //-----------------------------------------------------------------------
    $scope.clearForm = function() {
        $scope.newUser = angular.copy(defaultNewUser);
        $scope.AddUserForm.$setPristine();
        document.getElementById("clearFormBtnId").focus();
        dbRequestService.dbRequest($scope.usersDataGridOptions.data, EnumDbCmds.USERS);
    };

    //-----------------------------------------------------------------------
    // watchCollection companyList
    // 
    // When the user has access, a list of oganizations are returned, a 
    // company list. The current user level is stored in root scope that is 
    // used to dynamically update the list of user levels to assigned to
    // the new user being added.
    //-----------------------------------------------------------------------
    $scope.$watchCollection('companyList', function(newValue){
        if(newValue.length > 0)
        {
            if($rootScope.Level !== EnumUserLevels.USER)
            {               
               $scope.userLevels.push({text: 'Manager', 
                                       value: EnumUserLevels.MANAGER});
            };
           
            if($rootScope.Level === EnumUserLevels.ADMIN)
            {
               $scope.userLevels.push({text: 'Administrator', 
                                       value: EnumUserLevels.ADMIN});
            };
         };        
    });


    //-----------------------------------------------------------------------
    //                              Web Service 
    //-----------------------------------------------------------------------
    dbRequestService.dbRequest($scope.usersList, EnumDbCmds.USERS);
    dbRequestService.dbRequest($scope.companyList, EnumDbCmds.ORGKEYS, "", "");

});