/*******************************************************************************
* 
*    ULTRA STEREO LABS CONFIDENTIAL
*    _____________________________________________________________________
* 
*       Copyright (c) 2014 - 2015 Ultra Stereo Labs Incorporated 
*                         All Rights Reserved.
* 
*    NOTICE:  All information contained herein is, and remains
*    the property of Ultra Stereo Labs Incorporated and its suppliers,
*    if any.  The intellectual and technical concepts contained
*    herein are proprietary to Ultra Stereo Labs Incorporated
*    and its suppliers and may be covered by U.S. and Foreign Patents,
*    in process, and are protected by trade secret or copyright law.
*    Dissemination of this information or reproduction of this material
*    is strictly forbidden unless prior written permission is obtained
*    from Ultra Stereo Labs Incorporated. 181 Bonetti Drive, San Luis Obispo, 
*    California 93401-7397, USA – www.uslinc.com
*
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
*
*******************************************************************************/
//---------------------------------------------------------------------------
// ModifyUserController
//
// This controller requests all user data items and and allows updateds to
// entries based on authority level.  Level 3 can change all, level 2 can change
// only that level and lower.  Level 0 is not permitted to see this screen.
//---------------------------------------------------------------------------
app.controller('modifyUserController',
    function ($scope, $log, $filter, $rootScope, dbRequestService, dbUpdateService, dbDeleteService, createDialog) {
        //-------------------------------------------------------------------
        //                           Data
        //-------------------------------------------------------------------                           
        // Declare and object to use as the data source for the view binding.        
        $scope.userList = [];
        $scope.companyList = [];
        $scope.currentRow = -1;
        $scope.currentRowObject = "";
        $scope.isSelected = false;
        $scope.passChange = false;
        $scope.level = $rootScope.Level;
        $scope.saveResult = new ValueItemStruct();
        $scope.deleteResult = new ValueItemStruct();
            
        var createLists = true;    
        // Clear replaceable fields to prevent undefines later
        $scope.modify = {
            UserName: "",
            Email: "",
            Company: "",
            NotificationType: "",
            UserLevel: "",
            UID: "",
            Created: "",
            Phone: ""
        };
           
        //---------------------------------------------------------------
        // Data Grid Cell Template statusTemplate
        //
        // The userLevel template calls the converter function that 
        // returns a respective string for the enumerated user types.
        //---------------------------------------------------------------
        $scope.states = {
            valueToName: function(val, valueType) {
                var result = '';
                if(valueType ===0)
                {
                    if(val > EnumUserLevels.ADMIN  || val < EnumUserLevels.USER){
                        result = 'User';
                    }else{
                        result = $scope.userLevels[val].text;
                    }
                }else if(valueType===1){
                    if(val > EnumNotifyLevels.ALL || val < EnumNotifyLevels.NONE){
                        result = 'None';
                    }else{
                        result = $scope.notifyLevels[val].text;
                    }
                }
                    return result;
            },                
        };
        var levelTemplate = '<div style="padding-top:5px; margin-left:5px">{{grid.appScope.states.valueToName(row.entity.Level, 0)}}</div>';
        var notifyTemplate = '<div style="padding-top:5px; margin-left:5px">{{grid.appScope.states.valueToName(row.entity.Notify, 1)}}</div>';
            
        //---------------------------------------------------------------
        //                   Users Data Grid Definition
        //---------------------------------------------------------------
        $scope.gridOptions = {
            data: $scope.userList,
            enableRowSelection: true,
            enableSelectAll: false,
            multiSelect: false,
            selectionRowHeaderWidth: 35,
            rowHeight: 35,
            height: 35 * 5,
            minRowsToShow:10,
            showGridFooter: false,
            enableHorizontalScrollbar: 0, // for Scrollbars: NEVER: 0, ALWAYS: 1, WHEN_NEEDED: 2
            enableVerticalScrollbar: 0,
            enablePaginationControls: 1,
            paginationPageSize: 9,
            paginationPageSizes: [3, 6, 9],
            columnDefs: [
                {name: 'UID', displayName: 'User ID', enableHiding: false, width: '80'},
                {name: 'Name', displayName: 'User Name', enableHiding: false},
                {name: 'Company', displayName: 'Company', showHeader: false},
                {name: 'Email', displayName: 'Email Address', enableHiding: false},
                {name: 'NotificationType', displayName: 'Notification', showHeader: false, cellTemplate: notifyTemplate},
                {name: 'Created', displayName: 'Created', showHeader: false},
                {name: 'Level', displayName: 'Level', showHeader: false, cellTemplate: levelTemplate},
                {name: 'Phone', width: '160', enableHiding: false}]
        };
        // Do a request for all the data
        dbRequestService.dbRequest($scope.gridOptions.data, EnumDbCmds.USERS);
           
        //-----------------------------------------------------------------------
        // User Level
        // 
        // Create a list for user access levels, the default is only user.
        //-----------------------------------------------------------------------
        $scope.userLevels = [
            {
                text: 'User',
                value: EnumUserLevels.USER
            }];
     
        //-----------------------------------------------------------------------
        // Notification Levels
        // 
        // Create a list of items for the dropdown.
        //-----------------------------------------------------------------------
        $scope.notifyLevels = [
            {
                text: 'None',
                value: EnumNotifyLevels.NONE
            }, {
                text: 'Errors Only',
                value: EnumNotifyLevels.ERRORSONLY
            }, {
                text: 'All',
                value: EnumNotifyLevels.ALL
            }
        ];
                
        // Initialize the drop downs.
        $scope.modify.NotificationType = EnumNotifyLevels.NONE;
        $scope.modify.UserLevel = EnumUserLevels.USER;
            
        //---------------------------------------------------------------
        // watchCollection userList
        //
        // When there's a collection update the data grid object in the
        // page is not updated immediately. A timer function with the 
        // apply function for this component it is used to show/restore 
        // selection. The apply will start a digest that then updates all
        // bindings to the components within the page. 
        // Reselect the previously selected row (cleared on update)
        // Because there is no way to tell when the grid is completely 
        // rendered so the use of a timer after a list update event to
        // call apply.
        //---------------------------------------------------------------
        $scope.$watchCollection('userList', function(newValue){
            if(newValue.length > 0)
            {
                if(createLists)
                {
                    // dynamic list of companies.
                    $scope.companyList = [];
                    var companies = [];
                    for (var i in $scope.userList)
                    {
                        if($scope.userList[i].Company != null && $scope.userList[i].Company.length > 0)
                        {
                            item = new ListItemStruct();
                            item.value = $scope.userList[i].UID;
                            item.text  = $scope.userList[i].Company;                       
                            companies.push(item);
                        }
                    }
                    
                    $scope.companyList = $filter('unique')(companies, 'text');
                    //console.log("modifyUserController: Updating company list "+$scope.companyList.length);

                    // Update the user level assignment dropdown list, requires previledges
                    if($rootScope.Level !== EnumUserLevels.USER)
                    {               
                        $scope.userLevels.push({text: 'Manager', 
                                                value: EnumUserLevels.MANAGER});
                    };
           
                    if($rootScope.Level === EnumUserLevels.ADMIN)
                    {
                        $scope.userLevels.push({text: 'Administrator', 
                                                value: EnumUserLevels.ADMIN});
                    };
                    createLists = false;
                }    

                // There are no events from the grid to know when it has been 
                // updated, a call back timer is used to restore selected row.
                setTimeout(function () {
                    $scope.$apply(function () {
                        // console.log('modifyUserController: Received apply event.');
                        $scope.gridApi.selection.selectRow($scope.gridOptions.data[$scope.currentRow], true);
                    });
                }, 1000);
            }
        });                   
        
        //-------------------------------------------------------------------
        // $watch saveResult
        //
        // When the save action completes this event is signaled to update
        // the datagrid with the changes.
        //-------------------------------------------------------------------
        $scope.$watch('saveResult', function(newValue){
            if(newValue.Boolean === true){
                // Update the collection to show the change
                dbRequestService.dbRequest($scope.gridOptions.data, EnumDbCmds.USERS);
            }            
        }, true);
        //-------------------------------------------------------------------
        // saveChanges_Cmd
        // 
        // On click to save the current values in the input fields
        //-------------------------------------------------------------------
        $scope.saveChanges_Cmd = function () {
            $scope.saveResult.Boolean = false;
            var dataItem = new UsersStruct(
                    $scope.gridApi.selection.getSelectedRows()[0].UID,
                    $scope.modify.UserName,
                    $scope.modify.Company,
                    $scope.modify.NotificationType,
                    $scope.modify.UserLevel,
                    $scope.modify.Email,
                    $scope.modify.Phone,
                    $scope.gridApi.selection.getSelectedRows()[0].Created);
            // Save the changes with a results object to signal completion.
            dbUpdateService.dbUpdateWithResult(EnumDbUpdateCmds.USERS, 
                                               dataItem,
                                               $scope.saveResult);
        };
        //-------------------------------------------------------------------
        // fillAll
        // 
        // When the user/manager/administrator selects a user from the data
        // grid, the selected row items are used to populate the edit form.    
        //-------------------------------------------------------------------    
        $scope.fillAll = function (row) {
            // console.log("modifyUserController: filling in form "+ row.entity.Company);
            $scope.modify = {
                Email: row.entity.Email,
                UserName: row.entity.Name,
                Company: row.entity.Company,
                NotificationType: row.entity.Notify,
                UserLevel:row.entity.Level,
                UID: row.entity.UID,
                Created: row.entity.Created,
                Phone: row.entity.Phone,
                // Filling the passwords while they are hidden allows the form validate functions
                // to work correctly.  When the passwords are visible the rest of the form
                // is not editable and form validate doesn't work but we use password errors
                // to detect goodness of passwords because that is the only data saved at the time.
                Password: "*",
                password2: "*"
            };            
        };
            
        //-------------------------------------------------------------------
        // clearAll
        // 
        // Set the form to its prestine state, new.
        //-------------------------------------------------------------------
        $scope.clearAll = function () {
            $scope.modify = {
                UserName: "",
                Email: "",
                Company: "",
                NotificationType: 0,
                UserLevel: 0,
                UID: "",
                Created: "",
                Phone: "",
                Password: "",
                password2: ""
            };
        };
            
        //-------------------------------------------------------------------
        // confirmDelete_Cmd
        // 
        // Creates a confirm deletion popup dialog
        //-------------------------------------------------------------------
        $scope.confirmDelete_Cmd = function () {
            createDialog({
                id: 'simpleDialog',
                template:
                        '<div class="dialog-confirm">' +
                        ' <h3 class="dialog-header-confirm">Do you wish to delete the selected user?</h3>' +
                        '</div>',
                title: 'Delete Selected User',
                backdrop: true,
                modalClass: "modal",
                cancel: {label: 'Cancel', fn: null},
                success: {label: 'Delete', fn: function () {
                    $scope.deleteSelectedRecord();
                }}
            });
        };
        
        //-------------------------------------------------------------------
        // $watch deleteResult
        //
        // Event the is signaled when the deletion completes, time to 
        // refresh the data grid.
        //-------------------------------------------------------------------
        $scope.$watch('deleteResult', function(newValue){
            if(newValue.Boolean === true)
            {
                $scope.currentRow = -1;
                // Clear the input fields.                                               
                $scope.clearAll();
               // Update the collection to show the change
                dbRequestService.dbRequest($scope.gridOptions.data, EnumDbCmds.USERS);
            }           
        },true)
        
        //-------------------------------------------------------------------
        // deleteSelectedRecord
        // 
        // On confirmation from the user the selected record in the data
        // grid is delected.
        //-------------------------------------------------------------------        
        $scope.deleteSelectedRecord = function () {
            $scope.deleteResult.Boolean = false;
            
            dataItem = new UsersStruct();
            dataItem.Name = $scope.gridOptions.data[$scope.currentRow].Name;
            dataItem.Email = $scope.gridOptions.data[$scope.currentRow].Email;

            dbDeleteService.dbDeleteWithResult(EnumDbDeleteCmds.USER, 
                                               dataItem, 
                                               $scope.deleteResult);
        };
        
        /*******************************************************************
        * 
        * changePassword() - On a change password we only want the user
        * playing with the password. So on the HTML we disbale all other
        * data and focus on Password and Password Confirmation then allow
        * a save changes.
        * 
        *******************************************************************/
        $scope.changePassword_Cmd = function () {
            $scope.passwordChange = !$scope.passwordChange;
            if ($scope.passwordChange === true) {
                // Refill the grid incase the user changed anything before
                // deciding to change the password.  We don't use the data
                // and the fields are disabled but we want the user to know
                // current data remains the same.
                $scope.fillAll($scope.currentRowObject);
                // Clear the safety characters from the passwords.
                $scope.modify.Password = "";
                $scope.modify.password2 = "";
            }
        };
        
        //-------------------------------------------------------------------
        // onRegisterApi
        // 
        // Registered user interface operation with the data grid.
        //-------------------------------------------------------------------
        $scope.gridOptions.onRegisterApi = function (gridApi) {
            //set gridApi on scope
            $scope.gridApi = gridApi;
            gridApi.selection.on.rowSelectionChanged($scope, function (row) {
                $scope.currentRow = $scope.gridOptions.data.indexOf(row.entity);
                $scope.isSelected = row.isSelected;
                $scope.currentRowObject = row;
                if (row.isSelected) {
                    $scope.fillAll(row);
                } else {
                    $scope.clearAll();
                }
            });
            gridApi.selection.on.rowSelectionChangedBatch($scope, function (rows) {
                var msg = 'rows changed ' + rows.length;
            });
        };
    
    });
