CREATE DATABASE IF NOT EXISTS lssdb;

USE lssdb;

CREATE TABLE IF NOT EXISTS `RawData` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `OrgKey` varchar(50),
  `RecordNum` int(11),
  `DateTime` int(11),
  `Description` varchar(50),
  `Value` float,
  `Units` varchar(25),
  `Minimum` float,
  `Maximum` float,
  `OutsideLimits` tinyint(4),
  `SerialNumber` int(11),
  `TheaterName` varchar(50),
  `TheaterNumber` int(11),
  `AuditoriumNumber` int(11),
  `Comments` varchar(50),
  `Firmware` int(11),
  `Hardware` char(1),
  `IpAddress` varchar(50),
   PRIMARY KEY (`id`)
)ENGINE=InnoDB  DEFAULT CHARSET=latin1 AUTO_INCREMENT=1 ;

--
-- Table structure for table `Users`
--

CREATE TABLE IF NOT EXISTS `users` (
  `UID` int(11) NOT NULL AUTO_INCREMENT,
  `UserName` varchar(50) NOT NULL,
  `Password` varchar(200) NOT NULL,
  `Company` varchar(50) NOT NULL,
  `Level` int(2) NOT NULL DEFAULT 0,
  `NotificationType` int(2) NOT NULL,
  `Email` varchar(50) NOT NULL,
  `Phone` varchar(100) NOT NULL,
  `Created` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (`UID`)
) ENGINE=InnoDB  DEFAULT CHARSET=latin1 AUTO_INCREMENT=1 ;

CREATE TABLE IF NOT EXISTS `groups` (
  `GID` int(11) NOT NULL AUTO_INCREMENT,
  `PID` int(11) NOT NULL DEFAULT 0,
  `GroupName` varchar(50) NOT NULL,
  `OrgKey` varchar(50) NOT NULL,
  PRIMARY KEY (`GID`)
) ENGINE=InnoDB  DEFAULT CHARSET=latin1 AUTO_INCREMENT=1 ;

CREATE TABLE IF NOT EXISTS `usergroups` (
  `UGID` int(11) NOT NULL AUTO_INCREMENT,
  `UserID` int(11) NOT NULL,
  `GroupID` int(11) NOT NULL,
  `Active` int(3) NOT NULL,
  PRIMARY KEY (`UGID`)
) ENGINE=InnoDB  DEFAULT CHARSET=latin1 AUTO_INCREMENT=1 ;

CREATE TABLE IF NOT EXISTS `theatergroups` (
  `TGID` int(11) NOT NULL AUTO_INCREMENT,
  `TheaterNumber` int(11) NOT NULL,
  `OrgKey` varchar(50) NOT NULL,
  `GroupID` int(11) NOT NULL,
  PRIMARY KEY (`TGID`)
) ENGINE=InnoDB  DEFAULT CHARSET=latin1 AUTO_INCREMENT=1 ;

--
-- Email Settings and notifications 
--
CREATE TABLE IF NOT EXISTS mailsettings (
    ServerType int(8) NOT NULL DEFAULT 1,
    SMTPAuth BOOLEAN NOT NULL DEFAULT FALSE,
    SMTPSecure varchar(5) NOT NULL DEFAULT 'tls',
    HostName varchar(50) NOT NULL DEFAULT 'localhost.com',
    Port int(11) NOT NULL DEFAULT 25, 
    Username varchar(50) NOT NULL DEFAULT 'root@localhost.com',
    Password varchar(50) NOT NULL DEFAULT 'password'
) ENGINE=InnoDB  DEFAULT CHARSET=latin1 ;

CREATE TABLE IF NOT EXISTS `notifications` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `RawDataId` int(11) NOT NULL DEFAULT '0',
  `ReceivedDateTime` int(11) NOT NULL DEFAULT '0',
  `Email` varchar(50) NOT NULL DEFAULT "",
  `SentDateTime` int(11) NOT NULL DEFAULT '0',
  PRIMARY KEY (`id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1 AUTO_INCREMENT=1 ;

CREATE TABLE IF NOT EXISTS `notificationadmin` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `StatusText` text NOT NULL,
  `Running` int(8) NOT NULL DEFAULT '0',
  `LastCronRun` varchar(50) NOT NULL DEFAULT 'none',
  `Errors` int(11) NOT NULL DEFAULT '0',
  `IntervaluSecs` int(11) NOT NULL DEFAULT '1000',
  `DomainNameServer` varchar(50) NOT NULL DEFAULT 'lssdb.uslinc.com',
  `FromAddress` varchar(50) NOT NULL DEFAULT 'root@localhost.com',
  `FromName` varchar(50) NOT NULL DEFAULT 'LSS Web Server', 
  `LastSent` int(11) NOT NULL DEFAULT '0',
  PRIMARY KEY (`id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1 AUTO_INCREMENT=1 ;

--
-- Insert SysAdmin and Manager records to start the database
--

INSERT INTO `users` (`UID`, `UserName`, `Email`, `Phone`, `Password`,`Company`, `Level`,`NotificationType`, `Created`) VALUES
(1, 'System Administrator', 'admin', '1234567890', '$2a$10$c90a864d61a11ca7dbc7au1NO2QvYc3kLdoFeaapopwY90o/2JhHO','USL Inc.','2','0','2015-02-05 00:00:00'),
(2, 'Manager', 'manager', '0987654321', '$2a$10$807c41f29ae85e3730634eMQMgEiXBkfLRachbsluNadXdo21WFgG','USL Inc.','1','0','2015-02-05 00:00:00');

INSERT INTO `groups` (`GID`, `GroupName`,`OrgKey`,`PID`) VALUES
(1, 'US', 'uslinc.com', '0'),
(2, 'East Coast', 'uslinc.com', '1'),
(3, 'Central', 'uslinc.com' ,'1'),
(4, 'West Coast', 'uslinc.com', '1'),
(5, 'Vermont', 'uslinc.com', '2'),
(6, 'New York', 'uslinc.com' ,'2'),
(7, 'MaxWay Theaters', 'uslinc.com', '5'),
(8, 'Show Time Theaters', 'uslinc.com', '5'),
(9, 'NYC', 'uslinc.com' ,'6'),
(10, 'Queens', 'uslinc.com', '6'),
(11, 'Illinois', 'uslinc.com', '3'),
(12, 'Minnesota', 'uslinc.com' ,'3'),
(13, 'California', 'uslinc.com', '4'),
(14, 'Washington', 'uslinc.com' ,'4'),
(15, 'LA area Westwood Theaters', 'uslinc.com', '13'),
(16, 'OC area Orange Theaters', 'uslinc.com', '13'),
(17, 'Global Theater Chain', 'uslinc.com' ,'14'),
(18, 'Pacific Theater Chain', 'uslinc.com', '14'),
(19, 'CinemaWest Corp', 'uslinc.com', '0'),
(20, 'Village Theater Chain', 'uslinc.com', '19'),
(21, 'White Chain', 'uslinc.com' ,'19' ),
(22, 'Premier Theaters', 'uslinc.com', '19'),
(23, 'Calway', 'uslinc.com', '20'),
(24, 'Park', 'uslinc.com' ,'21'),
(25, 'Good Value', 'uslinc.com', '22'),
(26, 'ShowTime', 'uslinc.com', '20'),
(27, 'Greatway', 'uslinc.com', '21'),
(28, 'Eastman', 'uslinc.com', '22');


INSERT INTO `theatergroups` (`TheaterNumber`,`OrgKey`,`GroupID`) VALUES
(1,'USL-QA',1);

INSERT INTO mailsettings VALUES (DEFAULT,DEFAULT,DEFAULT,DEFAULT,DEFAULT,DEFAULT,DEFAULT);

INSERT INTO notificationadmin VALUES (DEFAULT,"default",DEFAULT,DEFAULT,DEFAULT,DEFAULT,DEFAULT,DEFAULT,DEFAULT,DEFAULT);