<?php
/*******************************************************************************
* 
*    ULTRA STEREO LABS CONFIDENTIAL
*    _____________________________________________________________________
* 
*       Copyright (c) 2014 - 2015 Ultra Stereo Labs Incorporated 
*                         All Rights Reserved.
* 
*    NOTICE:  All information contained herein is, and remains
*    the property of Ultra Stereo Labs Incorporated and its suppliers,
*    if any.  The intellectual and technical concepts contained
*    herein are proprietary to Ultra Stereo Labs Incorporated
*    and its suppliers and may be covered by U.S. and Foreign Patents,
*    in process, and are protected by trade secret or copyright law.
*    Dissemination of this information or reproduction of this material
*    is strictly forbidden unless prior written permission is obtained
*    from Ultra Stereo Labs Incorporated. 181 Bonetti Drive, San Luis Obispo, 
*    California 93401-7397, USA – www.uslinc.com
*
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
*
*******************************************************************************/
//-----------------------------------------------------------------------
// classDatabase
// 
// Common database routines to access and manager the RawData and 
// Notification tables. This will include helper functions to retrieve
// user level and the then use their group Organization Key to all them
// to only manage those records that belong to them. 
//-----------------------------------------------------------------------
class  classDatabase{
    //-------------------------------------------------------------------
    //                          Data
    //-------------------------------------------------------------------
    // Public properties
    public $dbConnected = false;
    
    // Private variables
    var $mysqli;

    //-----------------------------------------------------------------------
    // Constructor
    //
    // Created a connection with the database as supplied by the parent.
    //-----------------------------------------------------------------------
    function __construct($sqli)
    {
        $this->mysqli = $sqli;
        
        if($this->mysqli !== null)
        {
            $this->dbConnected = true;
        } 
        
    }

    //-----------------------------------------------------------------------
    // GetRawDataStatistics
    //
    // Returns the number of items in the array with date of the newest and
    // oldest entries.
    //-----------------------------------------------------------------------
    public function GetRawDataStatistics()
    {
        // Default results
        $result['count']= 0;        
        $result['newestDate'] = 0;
        $result['oldestDate'] = 0;
        
        if($this->dbConnected)
        {
            
            // Get the number of records
            $query = 'SELECT COUNT(*) AS id FROM RawData';
            $value = $this->mysqli->query($query);
            $row = array();
            if( $value->num_rows > 0)   
            {  
               $row = $value->fetch_assoc();
               $result['count'] = $row['id'];
            }            
            
            // Get the newest record
            $query = 'SELECT DateTime FROM RawData ORDER BY DateTime DESC LIMIT 1';
            $value = $this->mysqli->query($query);
            $row = array();
            if( $value->num_rows > 0)   
            {  
               $row = $value->fetch_assoc();
               $result['newestDate'] = $row['DateTime'];
            }
            
            // Get the oldest record
            $query = 'SELECT DateTime FROM RawData ORDER BY DateTime ASC LIMIT 1';
            $value = $this->mysqli->query($query);
            $row = array();
            if( $value->num_rows > 0)   
            {  
               $row = $value->fetch_assoc();
               $result['oldestDate'] = $row['DateTime'];
            }
        }
        return $result;
    }
    
    //-----------------------------------------------------------------------
    // GetNotificationStatistics
    //
    // Returns the number of items in the array with date of the newest and
    // oldest entries.
    //-----------------------------------------------------------------------
    public function GetNotificationStatistics()
    {        
        $result['count']= 0;
        $result['newestDate'] = 0;
        $result['oldestDate'] = 0;
        
        if($this->dbConnected)
        {        
            // Get the number of records
            $query = 'SELECT COUNT(*) AS id FROM notifications';
            $value = $this->mysqli->query($query);            
            $row = array();
            if( $value->num_rows > 0)   
            {  
               $row = $value->fetch_assoc();
               $result['count'] = $row['id'];
            }            
            
            // Get the newest record
            $query = 'SELECT ReceivedDateTime FROM notifications ORDER BY ReceivedDateTime DESC LIMIT 1';
            $value = $this->mysqli->query($query);            
            $row = array();
            if( $value->num_rows > 0)   
            {  
               $row = $value->fetch_assoc();
               $result['newestDate'] = $row['ReceivedDateTime'];
            }
            
            // Get the oldest record
            $query = 'SELECT ReceivedDateTime FROM notifications ORDER BY ReceivedDateTime ASC LIMIT 1';
            $value = $this->mysqli->query($query);            
            $row = array();
            if( $value->num_rows > 0)   
            {  
               $row = $value->fetch_assoc();
               $result['oldestDate'] = $row['ReceivedDateTime'];
            }                           
        }
        return $result;
    }
 
    //-----------------------------------------------------------------------
    // GetRawDataAdmin
    //
    // Returns the complete data base entries with onlty those items of
    // interest to perform database management.
    // 
    //-----------------------------------------------------------------------
    public function GetRawDataAdmin()
    {
        $result = array();
        if($this->dbConnected)
        {
            $query = 'SELECT DISTINCT id, OrgKey, DateTime, TheaterNumber FROM RawData ORDER BY DateTime ASC';
            $value = $this->mysqli->query($query);

            if( $value->num_rows > 0)   
            {  
                while($row = $value->fetch_assoc()) {
                    $result[] = $row;	
                }
            }
        }
        return $result;
    }
    
    //-----------------------------------------------------------------------
    // DeleteRawData
    //
    // Using the specified Unix Epoch date range, remove the rows.
    //-----------------------------------------------------------------------
    public function DeleteRawData($StartTime, $EndTime)
    {        
        $query = "DELETE FROM RawData WHERE ".
                 "DateTime >=".$StartTime." AND DateTime <=".$EndTime;
     
        $result = $this->mysqli->query($query);
        if($result === true){
            return true;
        }else{ 
            return false;
        }        
    }
    
     //-----------------------------------------------------------------------
    // GetNotifications
    //
    // Returns the complete data base of items of notifications.
    //-----------------------------------------------------------------------
    public function GetNotifications()
    {
        $result = array();
        if($this->dbConnected)
        {
            $query = 'SELECT * FROM notifications ORDER BY ReceivedDateTime ASC';
            $value = $this->mysqli->query($query);

            if( $value->num_rows > 0)   
            {  
                while($row = $value->fetch_assoc()) {
                    $result[] = $row;	
                }
            }
        }
        return $result;
    }

    //-----------------------------------------------------------------------
    // DeleteNotifications
    //
    // Using the specified Unix Epoch date range, remove the rows. The 
    // SentDataTime also contains status values such as not sent.
    //-----------------------------------------------------------------------
    public function DeleteNotifications($StartTime, $EndTime)
    {        
        $Start = floatval($StartTime);
        $End   = floatval($EndTime);
        
        $query = "DELETE FROM notifications WHERE ".
                 "ReceivedDateTime >=".$Start." AND ReceivedDateTime <=".$End;
     
        $result = $this->mysqli->query($query);
        if($result === true){
            return true;
        }else{ 
            return false;
        }        
    }
    
    //-----------------------------------------------------------------------
    // GetNumPendingNotifications
    //
    // Return the number of SentDateTime = 0, a pending status.
    //-----------------------------------------------------------------------
    public function GetNumPendingNotifications()
    {
        $count = 0;        
        if($this->dbConnected)
        {
            $query = "SELECT COUNT(*) as Status FROM notifictions ".
                     "WHERE SentDateTime =".$DB_NOTIFICATION_NOT_SENT;
            $result = $this->mysqli->query($query);
            if($result->num_rows > 0)
            {
                $row = array();
                $row = $value->fetch_assoc();
                $count = $row['Status'];
            }
        }
        return $count;
    }
    
    //-----------------------------------------------------------------------
    // GetNumNotificationErrors()
    //
    // When SentDateTime contains a -1 or -2, they are counted. The count is
    // then returned.
    //-----------------------------------------------------------------------
    public function GetNumNotificationErrors()
    {
        $count = 0;        
        if($this->dbConnected)
        {
            $query = "SELECT COUNT(*) as Status FROM notifictions ".
                     "WHERE SentDateTime <".$DB_NOTIFICATION_NOT_SENT;
            $result = $this->mysqli->query($query);
            if($result->num_rows > 0)
            {
               $row = array();
               $row = $value->fetch_assoc();
               $count = $row['Status'];
            }
        }
        return $count;        
    }
    
    //-----------------------------------------------------------------------
    // GetNotificationErrorEntries
    //
    // Retrieves the notifications that are marked as errored.
    //-----------------------------------------------------------------------
    public function GetNotificationErrorEntries()
    {
        $arr = array();
        if($this->dbConnected)
        {
            $query = "SELECT * FROM notifications ".
                     "WHERE SentDateTime <".$DB_NOTIFICATION_NOT_SENT;
            $result = $this->mysqli->query($query);
            
            if($result->num_rows > 0)
            {
                while($row = $result->fetch_assoc()) {
                    $arr[] = $row;
                }
            }
        }
        return $arr;
    }
    
    //-----------------------------------------------------------------------
    // DeletePendingNotifications
    //
    // This will remove pending notifications, those that have not been 
    // sent.
    //-----------------------------------------------------------------------
    public function DeletePendingNotifications()
    {       
        $query = "DELETE FROM notifications WHERE ".
                 "SentDateTime = ".$DB_NOTIFICATION_NOT_SENT;
        $result = $this->mysqli->query($query);
        if($result->num_rows > 0){
            return true;
        }else{ 
            return false;
        }        
    }
    
    //-----------------------------------------------------------------------
    // GetCompanies
    // 
    // Returns a list of companies found in the users database table.
    //-----------------------------------------------------------------------
    public function GetCompanies()
    {
        $result = array();
        if($this->dbConnected)
        {
            $query = 'SELECT DISTINCT company FROM users ORDER BY company ASC';
            $value = $this->mysqli->query($query);

            if( $value->num_rows > 0)   
            {  
                while($row = $value->fetch_assoc()) {
                    if(strlen($row['company'])> 0){
                        $result[] = $row;	
                    }
                }
            }                           
        }
        return $result;
    }
    
}

?>
