<?php
    /************************************************************************
    * 
    *    ULTRA STEREO LABS CONFIDENTIAL
    *    ____________________________________________________________________
    * 
    *       Copyright (c) 2014 - 2015 Ultra Stereo Labs Incorporated 
    *                         All Rights Reserved.
    * 
    *    NOTICE:  All information contained herein is, and remains
    *    the property of Ultra Stereo Labs Incorporated and its suppliers,
    *    if any.  The intellectual and technical concepts contained
    *    herein are proprietary to Ultra Stereo Labs Incorporated
    *    and its suppliers and may be covered by U.S. and Foreign Patents,
    *    in process, and are protected by trade secret or copyright law.
    *    Dissemination of this information or reproduction of this material
    *    is strictly forbidden unless prior written permission is obtained
    *    from Ultra Stereo Labs Incorporated. 181 Bonetti Drive, San Luis Obispo, 
    *    California 93401-7397, USA – www.uslinc.com
    *
    *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
    *    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    *    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
    *    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
    *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
    *
    *
    *************************************************************************/
    //-----------------------------------------------------------------------
    // databaseAdmin
    // 
    // Database Administration service that is invoked from the page via
    // a service. A generic jason object is used for passing commands and
    // parameters to this server side service.
    // 
    // The following json structure is used:
    //  'command' = The command token
    //  'param1'  = Start Time Unix Epoch
    //  'param2'  = End Time Unix Epoch
    //
    // The RawData and notifications tables grow overtime at an rate that 
    // depends on the number of measurements and notification emails sent. 
    // This module will be used to perform maintenence and to manage the 
    // size of the database.
    // 
    //-----------------------------------------------------------------------
    // Check authentication of user and the request type, if not authenticated
    // the script will terminate execution with continuing to execute the 
    // following code.
    require_once 'authenticate.php';
    // When getting here the user and request has been authenticated
    // Get the Database configuration.
    include_once('../includes/config.php');
    // Check if user has access
    HasAccess(USER_ADMIN, true);
    
    include_once('classDatabase.php');

    // Create an instance of a database interface API.   
    $mysqli = new mysqli($DB_HOST, $DB_USER, $DB_PASS, $DB_NAME);
    $Database = new classDatabase($mysqli);
    
    // Retrieve the jason data passed in the post.
    $str_json = file_get_contents('php://input');
    // Convert the jason data to an associative array.
    $dataArray = json_decode($str_json, true);
    $resultArray = null;
    // Process the command
    switch ($dataArray['command']) 
    {
        case $DB_CMD_RAWDATA_STATISTICS:
            $resultArray = $Database->GetRawDataStatistics();
            ProcessResults($resultArray);
            break;
        case $DB_CMD_NOTIFICATION_STATISTICS:
            $resultArray = $Database->GetNotificationStatistics();
            ProcessResults($resultArray);
            break;
        case $DB_CMD_NOTIFICATIONS_PENDING_COUNT:
            $resultArray['Count'] = $Database->GetNumPendingNotifications();
            ProcessResults($resultArray);
            break;
        case $DB_CMD_NOTIFICATION_ERROR_COUNT:
            $resultArray['Count'] = $Database->GetNumNotificationErrors();
            ProcessResults($resultArray);
            break;
        case $DB_CMD_NOTIFICATION_ERROR_ENTRIES:
            $resultArray = $Database->GetNotificationErrorEntries();
            ProcessResults($resultArray);
            break;
        case $DB_CMD_NOTIFICATIONS_PENDING_DELETE:
            $resultArray['Result'] = $Database->DeletePendingNotifications();
            ProcessResults($resultArray);
            break;
        case $DB_CMD_RAWDATA_DELETE:
            $resultArray['Result'] = $Database->DeleteRawData($dataArray['param1'],$dataArray['param2']);
            ProcessResults($resultArray);
            break;
        case $DB_CMD_NOTIFICATIONS_DELETE:
            $resultArray['Result'] = $Database->DeleteNotifications($dataArray['param1'],$dataArray['param2']);
            ProcessResults($resultArray);
            break;
        case $DB_CMD_ADMIN_RAWDATA:
            $resultArray = $Database->GetRawDataAdmin();
            ProcessResults($resultArray);
            break; 
        case $DB_CMD_ADMIN_NOTIFICATIONS:
            $resultArray = $Database->GetNotifications();
            ProcessResults($resultArray);
            break; 
        case $DB_CMD_COMPANIES:
            $resultArray = $Database->GetCompanies();
            ProcessResults($resultArray);
            break;
        default:
            ProcessResults(null);
            break;
    }
    
    //-----------------------------------------------------------------------
    // ProcessResults
    //
    // This will return the results without further processing.
    //-----------------------------------------------------------------------
    function ProcessResults($result)
    {
        if($result == null)
        {
            $response = array();
            $response['status']  = "error";
            $response['message'] = "Failed to retrieve data, record not found.";
            // Encode the response
            $json_response = json_encode($response);
        }else{        
            # JSON-encode the response
            $json_response = json_encode($result);

        }
        // # Return the response
        echo $json_response;
    }
    
?>
