<?php
/*******************************************************************************
* 
*    ULTRA STEREO LABS CONFIDENTIAL
*    _____________________________________________________________________
* 
*       Copyright (c) 2014 - 2015 Ultra Stereo Labs Incorporated 
*                         All Rights Reserved.
* 
*    NOTICE:  All information contained herein is, and remains
*    the property of Ultra Stereo Labs Incorporated and its suppliers,
*    if any.  The intellectual and technical concepts contained
*    herein are proprietary to Ultra Stereo Labs Incorporated
*    and its suppliers and may be covered by U.S. and Foreign Patents,
*    in process, and are protected by trade secret or copyright law.
*    Dissemination of this information or reproduction of this material
*    is strictly forbidden unless prior written permission is obtained
*    from Ultra Stereo Labs Incorporated. 181 Bonetti Drive, San Luis Obispo, 
*    California 93401-7397, USA - www.uslinc.com
*
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
*
*******************************************************************************/
//---------------------------------------------------------------------------
// notification
//
// This is used for updating and sending out a notification when a post
// of LSS test results are received. When the cron process is stopped, a
// test result will need to generate an email with notification. This is 
// required for system demonstration. A cron process with do a simular task
// periodically in background as eith a batched process or timed scheduled.
//---------------------------------------------------------------------------
// If already include a warning is thrown and scrip continues to execute.
include_once('notifications/classNotification.php');

require 'PHPMailerAutoload.php';

//---------------------------------------------------------------------------
//                           Global Data
//---------------------------------------------------------------------------

$notify = null;

$usersArray = array();
$notificationArray = array();

//---------------------------------------------------------------------------
// NewNotificationInstance
//
// This creates a new instance of the notification class that contains the
// methods for accessing the database to retrieve and update items releated
// to executing the notifications. For instance there are users and user 
// groups involved, and then the emails that require multiple reports within
// one.
// 
// This parent module will have access to classes. The $NotifyAdmin can be
// declared in another parent, if not Declare here.
//---------------------------------------------------------------------------
function NewNotificationInstance($sqli)
{    
    global $notify;
    global $NotifyAdmin;
    global $mysqli;
    
    if($mysqli == null)
    {
        $mysqli = $sqli;
    }
    
    if($notify == null)
    {
        $notify = new classNotification($sqli);
    }
    
    if($NotifyAdmin == null)
    {
        $NotifyAdmin = new classAdmin($sqli);
    }
}

//---------------------------------------------------------------------------
// ProcessForNotification
//
// This will check the results and if required (failed) send an email and 
// update the notification's database. The notification table contains a 
// list of associated users with a post status. The status will indicate 
// a pending email to be sent, an email sent, an attempt to send but fialed,
// and a non-assigned user(s).
// 
//---------------------------------------------------------------------------
function ProcessForNotification($data, $recordId) {
    // Global variable references.
    global $notify;
    // User notification levels
    global $DB_NOTIFICATIONS_ERRORS;
    global $DB_NOTIFICATIONS_ALL;
    
    // Notification status
    global $DB_NOTIFICATION_NOT_SENT;
    global $DB_NOTIFICATION_NOT_ASSIGNED;
    
    // Local data
    $usersArray = array();
    $notificationArray = array();
    

    // Convert the values floating point type
    $val = (float) $data->Value;
    $max = (float) $data->Maximum;
    $min = (float) $data->Minimum;
    // Check if the value is out of limits.
    if (($val > $max) || ($val < $min)) {
        $valueOutOfLimits = true;
    }else{
        $valueOutOfLimits = false;        
    }
    
    
    $usersArray = $notify->GetUsersForReport($data);
    
    // For the LSS-100 posted results
    if(count($usersArray)>0)
    {
    
        $indx=0;
        // Check if send a notification on error or all.
        foreach ($usersArray as $user)
        {
           $notifyUser = intval($user['NotificationType']);
           if($notifyUser === $DB_NOTIFICATIONS_ALL)
           {
                $notificationArray[$indx]['RawDataId'] = $recordId;
                $notificationArray[$indx]['ReceivedDateTime'] = time();
                $notificationArray[$indx]['Email'] = $user['Email'];
                $notificationArray[$indx]['UserName'] = $user['UserName'];
                $notificationArray[$indx]['SentDateTime'] = $DB_NOTIFICATION_NOT_SENT;
                $indx++;
           }
           elseif ($notifyUser === $DB_NOTIFICATIONS_ERRORS && $valueOutOfLimits)
           {
                $notificationArray[$indx]['RawDataId'] = $recordId;
                $notificationArray[$indx]['ReceivedDateTime'] = time();
                $notificationArray[$indx]['Email'] = $user['Email'];
                $notificationArray[$indx]['UserName'] = $user['UserName'];
                $notificationArray[$indx]['SentDateTime'] = $DB_NOTIFICATION_NOT_SENT;
                $indx++;                      
           }
        } 
        
        // Just in case...
        if($indx>0)
        {
            // Need to update time stamp at time when sending email
            //$notificationArray[$indx]['SentDateTime'] = time();
            $notify->SaveNotification($notificationArray);
        }

    }else{
        // When there are no users for this report then create a 
        // notification with respective status.
        $notificationArray[0]['RawDataId'] = $recordId;
        $notificationArray[0]['ReceivedDateTime'] = time();
        $notificationArray[0]['Email'] = "Unknown";
        $notificationArray[0]['UserName'] = "Unknown";
        $notificationArray[0]['SentDateTime'] = $DB_NOTIFICATION_NOT_ASSIGNED;
        $notify->SaveNotification($notificationArray);
    }
    

}

//---------------------------------------------------------------------------
// SetTheaterInfoHtml
//
// When processing the notification data for multiple theater, the email
// needs to contains information about each location. This creates the 
// header data.
//---------------------------------------------------------------------------
function SetTheaterInfoHtml($data, $notificationLevel)
{
    $reportTime = intval($data['DateTime']);
    $timeStamp = new DateTime("@$reportTime");
    $msgBodyHeader = "LSS-100 Notification ".$timeStamp->format('Y-m-d H:i:s')." UTC";
        
    // Create the email message that contains multiple results for this user(email).
    $htmlTheaterInfo = "<h3><Font color=\"blue\">$msgBodyHeader</Font></h3><br>".
                       "<table width='40%'><tbody>".
                       "<col style='width:10%'>\n".
                       "<col style='width:20%'>\n".
                       "<tr><td style='text-align:right'><b>Notification Level:</b></td>\n".
                       "<td>".$notificationLevel."</td></tr>".
                       "<tr><td style='text-align:right'><b>Company:</b></td>\n".
                       "<td>".$data['OrgKey']."</td></tr>".
                       "<tr><td style='text-align:right'><b>Theater Name:</b></td>\n".
                       "<td>".$data['TheaterName']."</td></tr>".
                       "<tr><td style='text-align:right'><b>Theater Number:</b></td>\n".
                       "<td>".$data['TheaterNumber']."</td></tr>".
                       "<tr><td style='text-align:right'><b>Screen:</b></td>\n".
                       "<td>".$data['AuditoriumNumber']."</td></tr>".
                       "<tr><td style='text-align:right'><b>Serial Number:</b></td>\n".
                       "<td>".$data['SerialNumber']."</td></tr>\n".
                       "<tr><td style='text-align:right'><b>Comments:</b></td>\n".
                       "<td>".$data['Comments']."</td></tr>\n".
                       "<tr><td style='text-align:right'><b>Firmware:</b></td>\n".
                       "<td>".$data['Firmware']."</td></tr>\n".
                       "</tbody></table>\n".
                       "<table width='90%'><caption><b>Report Data</b></caption>\n".
                       "<col style='width:20%'>\n".
                       "<col style='width:20%'>\n".
                       "<col style='width:20%'>\n".
                       "<col style='width:10%'>\n".
                       "<col style='width:10%'>\n".
                       "<col style='width:10%'>\n".
                       "<col style='width:10%'>\n".
                       "<col style='width:10%'>\n".
                       "<col style='width:10%'>\n".
                       "<thead><tr>\n".
                       "<th style='text-align:left'>Date Time</th>\n".
                       "<th>Description</th>\n".
                       "<th>Value</th>\n".
                       "<th>Minimum</th>\n".
                       "<th>Maximum</th>\n".
                       "<th>Record</th>\n".
                       "<th>Data ID</th>\n".
                       "<th>Link</th>\n".
                       "</tr></thead>\n".
                       "<tbody><tr>\n";
   
    return $htmlTheaterInfo;
}

//---------------------------------------------------------------------------
// SetTheaterInfoText
// 
// Since text is limited we have only one email at a time, this is 
// a unique case. Data limit imposed.
//---------------------------------------------------------------------------
function SetTheaterInfoText($data)
{
    $reportTime = intval($data['DateTime']);
    $timeStamp = new DateTime("@$reportTime");
    
    // For a text message limit the number of character. 
    $textTheaterInfo =$timeStamp->format('Y-m-d H:i:s')." UTC".
                      " Company:".$data['OrgKey'].
                      " Theater:".$data['TheaterName'].
                      " Screen:".$data['AuditoriumNumber'].
                      " Description:".$data['Description']. 
                      " Value:".$data['Value'].$data['Units'].
                      " Min:".$data['Minimum']." Max:".$data['Maximum'];

    return $textTheaterInfo;
}

//---------------------------------------------------------------------------
// SendNotificationMail
//
// A list of notifications for each user is created. Then each user in the
// list will be sent an email. 
//---------------------------------------------------------------------------
function SendNotificationMail() {
    // Global variable references.
    global $notify;
    global $mysqli;
    
    // User notificiation selection/type
    global $DB_NOTIFICATIONS_ALL;
    global $DB_NOTIFICATIONS_ERRORS;
    // Notification status 0 not sent otherwise
    global $DB_NOTIFICATION_FAILED;
    
    global $NotifyAdmin;
    
    $numOfEmailsSent = 0;
    // Time span for email links
    $epochFourDays = 345600;
    
    $msgHtml = "";
    $msgTxt = "";
    
    // This will get a list of notifications that has yet to be 
    // sent with a DataTimeSent of zero, indicating pending.
    $notificationList = $notify->GetEmails();
    while(count($notificationList))
    {
        
        $data = $notify->GetRawDataRecord(intval($notificationList[0]['RawDataId']));
        if($data === null)
        {
            $notify->UpdateNotificationFailedStatus(intval($notificationList[0]['RawDataId']), $DB_NOTIFICATION_FAILED);
            $notificationList = $notify->GetEmails();
            continue;
        }
        
        $EmailAddress = $notificationList[0]['Email'];
        $userInfo = $notify->GetUserInfo($EmailAddress);
        $EmailUserName = $userInfo['UserName'];
        $NotificationLevel = $userInfo['NotificationType'] == $DB_NOTIFICATIONS_ALL ? "All":"Errors Only" ;
               
        $emailBodyHtml = SetTheaterInfoHtml($data, $NotificationLevel);
        $emailBodyText = SetTheaterInfoText($data);
        $emailLink = null; 
        // Get the current company, theater, and screen.
        $currentOrgKey = $data['OrgKey'];
        $currentTheaterNumber = intval($data['TheaterNumber']);
        $currentAuditoriumNumber = intval($data['AuditoriumNumber']);
                
        foreach($notificationList as $reportItem)
        {
            $data = $notify->GetRawDataRecord(intval($reportItem['RawDataId']));
            if($data === null)
            {
                continue;
            }
            // Check if the report belongs to the same company, theater, and screen.
            if($currentOrgKey !== $data['OrgKey'] || 
               $currentTheaterNumber !== intval($data['TheaterNumber'])||
               $currentAuditoriumNumber != intval($data['AuditoriumNumber']))
            {
                $emailBodyHtml .= "</tbody></table><br>";
                
                // Update for new theater
                $emailBodyHtml .= SetTheaterInfoHtml($data, $NotificationLevel);  
                $currentOrgKey = $data['OrgKey'];
                $currentTheaterNumber = intval($data['TheaterNumber']);
                $currentAuditoriumNumber = intval($data['AuditoriumNumber']);
            }
            
            // Convert the values floating point type
            $val = floatval($data['Value']);
            $max = floatval($data['Maximum']);
            $min = floatval($data['Minimum']);
            // Check if the value is out of limits.
            if (($val > $max) || ($val < $min)) {
                $valueOutOfLimits = true;
            }else{
                $valueOutOfLimits = false;        
            }
            
            $reportTime = intval($data['DateTime']);
            $timeStamp = new DateTime("@$reportTime");
            
            $encodedDescription = urlencode($data['Description']);
            $emailLink  = "<a href=\"http://$NotifyAdmin->DomainNameServer:80?".
                          "loc=screens&orgkey=".$data['OrgKey'].
                          "&theater=".$data['TheaterNumber'].
                          "&screen=".$data['AuditoriumNumber'].
                          "&id=".$data['id'].
                          "&description=".$encodedDescription.
                          "&startdate=".((string)(($reportTime-$epochFourDays)*1000)).
                          "&enddate=".((string)(($reportTime+$epochFourDays)*1000))."\">View</a>";           
            
            $emailBodyHtml.="<tr>";
            if($valueOutOfLimits){
                $emailBodyHtml.="<td><Font color=\"red\">".$timeStamp->format('Y-m-d H:i:s')."</Font></td>\n";
                $emailBodyHtml.="<td style='text-align:center'><Font color=\"red\">".$data['Description']."</Font></td>\n";
                $emailBodyHtml.="<td style='text-align:center'><Font color=\"red\">".$data['Value'].$data['Units']."</Font></td>\n";
                $emailBodyHtml.="<td style='text-align:center'><Font color=\"red\">".$data['Minimum']."</Font></td>\n";
                $emailBodyHtml.="<td style='text-align:center'><Font color=\"red\">".$data['Maximum']."</Font></td>\n";
                $emailBodyHtml.="<td style='text-align:center'><Font color=\"red\">".$data['RecordNum']."</Font></td>\n";
                $emailBodyHtml.="<td style='text-align:center'><Font color=\"red\">".$data['id']."</Font></td>\n";                
            }else{                
                $emailBodyHtml.="<td>".$timeStamp->format('Y-m-d H:i:s')."</td>\n";
                $emailBodyHtml.="<td style='text-align:center'>".$data['Description']."</td>\n";
                $emailBodyHtml.="<td style='text-align:center'>".$data['Value'].$data['Units']."</td>\n";
                $emailBodyHtml.="<td style='text-align:center'>".$data['Minimum']."</td>\n";
                $emailBodyHtml.="<td style='text-align:center'>".$data['Maximum']."</td>\n";
                $emailBodyHtml.="<td style='text-align:center'>".$data['RecordNum']."</td>\n";
                $emailBodyHtml.="<td style='text-align:center'>".$data['id']."</td>\n";
            }
            
            $emailBodyHtml.="<td style='text-align:center'>".$emailLink."</td>\n";
            $emailBodyHtml.="</tr>\n";
        }
        
        $emailBodyHtml .= "</tbody></table><br>";
        
        // New instance of an email otherwise it will combine them into one.
        $mail = new PHPMailer;
        
        // Get the Email settings.
        $query = "SELECT * FROM mailsettings ORDER BY ServerType LIMIT 1";
        $result = $mysqli->query($query);
        if ($result != null && $result->num_rows > 0) {
            // Retrieve the data from the query result
            $setting = array();
            $setting = $result->fetch_assoc();

            switch ($setting['ServerType']) {
                case 0:
                    // Set mailer to use SMTP 
                    $mail->isSMTP();
                    break;
                case 1:
                    // Set mailer to use php mail 
                    $mail->isMail();
                    break;
                case 2:
                    // Set mailer to use Sendmail 
                    $mail->isSendmail();
                    break;
                case 3:
                    // Set mailer to use Qmail 
                    $mail->isQmail();
                    break;
                default:
                    return;
                    break;
            }
            // SMTP authentication enable setting
            $mail->SMTPAuth = $setting['SMTPAuth'];
            // tls or ssl encryption
            $mail->SMTPSecure = $setting['SMTPSecure'];
            // set the SMTP server, lss web server's address
            $mail->Host = $setting['HostName'];
            // TCP port to connect to, SMTP server
            $mail->Port = $setting['Port'];

            // This application's user information
            // SMTP server/client username
            $mail->Username = $setting['Username'];
            // SMTP server/client password
            $mail->Password = $setting['Password'];
        } else {

            // The following settings are the defaults.....
            // $mail->isSMTP();     // Set mailer to use SMTP
            $mail->isMail();        // Set mailer to use php mail
            // $mail->isSendmail(); // Set mailer to use Sendmail
            // $mail->isQmail();    // Set mailer to use Qmail

            $mail->SMTPDebug = 0;   // 0 = no output, 1 = errors and messages, 2 = messages only.

            $mail->SMTPAuth = false;                    // SMTP authentication enable setting
            $mail->SMTPSecure = 'tls';                  // Enable TLS encryption, `ssl` also accepted
            $mail->Host = 'localhost.com';              // set the SMTP server
            $mail->Port = 25;                           // TCP port to connect to, SMTP server
            // This application's user information
            $mail->Username = 'root@localhost.com';     // SMTP server/client username
            $mail->Password = 'secret';                 // SMTP server/client password
        }

        // Email data from the service call
        // This will need to identify the user or LSS daemon.
        $mail->From = $NotifyAdmin->FromAddress;
        $mail->FromName = $NotifyAdmin->FromName;

        $mail->isHTML(true);
        $mail->Subject = 'LSS-100P Notification';
        $mail->Body = $emailBodyHtml;
        $mail->AltBody = $emailBodyText;

        $mail->addAddress($EmailAddress, $EmailUserName);
        if ($mail->send()) 
        {            
            foreach($notificationList as $reportItem)
            {
                $notify->UpdateNotificationStatus(intval($reportItem['RawDataId']), $EmailAddress, time());
            }
            $numOfEmailsSent++;
        }
        else
        {
            foreach($notificationList as $reportItem)
            {
                $notify->UpdateNotificationStatus(intval($reportItem['RawDataId']), $EmailAddress, $DB_NOTIFICATION_FAILED);
            }
        }
        
        // Get the pending emails for the next user.
        $notificationList = $notify->GetEmails();
        // Delay between sending emails.
        if(count($notificationList))
        {
            usleep ($NotifyAdmin->IntervaluSecs);
        }
        
    }
    // Number of emails sent status.
    return $numOfEmailsSent;
}

?>