<?php
/*******************************************************************************
* 
*    ULTRA STEREO LABS CONFIDENTIAL
*    _____________________________________________________________________
* 
*       Copyright (c) 2014 - 2015 Ultra Stereo Labs Incorporated 
*                         All Rights Reserved.
* 
*    NOTICE:  All information contained herein is, and remains
*    the property of Ultra Stereo Labs Incorporated and its suppliers,
*    if any.  The intellectual and technical concepts contained
*    herein are proprietary to Ultra Stereo Labs Incorporated
*    and its suppliers and may be covered by U.S. and Foreign Patents,
*    in process, and are protected by trade secret or copyright law.
*    Dissemination of this information or reproduction of this material
*    is strictly forbidden unless prior written permission is obtained
*    from Ultra Stereo Labs Incorporated. 181 Bonetti Drive, San Luis Obispo, 
*    California 93401-7397, USA - www.uslinc.com
*
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
*
*******************************************************************************/
//-----------------------------------------------------------------------
// classAdmin
//
// Queries the notification admin database table to make settings 
// available.
//-----------------------------------------------------------------------
class classAdmin {
    //-------------------------------------------------------------------
    //                          Data
    //-------------------------------------------------------------------
    // Public properties
    public $Id = 0;
    public $StatusText = "";
    public $Running = 0;
    public $LastCronRun = "";
    public $Errors = 0;
    public $IntervaluSecs = 0;
    public $DomainNameServer = "";
    public $FromAddress ="";
    public $FromName = "";
    public $LastSent = 0;
    public $DataValid = false;
    
    // Private variables
    var $mysqli;
    
    //-----------------------------------------------------------------------
    // Constructor
    //
    // Initializes the connection with the database to perform updates
    // and retrieve settings.
    //-----------------------------------------------------------------------
    function __construct($sqli)
    {
        //echo("**************** classAdmin Constructor ********************");
        $this->mysqli = $sqli;
        
        if($this->mysqli !== null)
        {
            $this->UpdateProperties();
        }
    }    
    
    //-----------------------------------------------------------------------
    // UpdateProperties                       
    //
    // Retrieves the settings from the database and updates the public
    // properties.
    //-----------------------------------------------------------------------
    function UpdateProperties()
    {
        $result = null;    
        $query="SELECT DISTINCT id,StatusText,Running,LastCronRun,Errors,".
               "IntervaluSecs,DomainNameServer, FromAddress,FromName,LastSent ".
               "FROM notificationadmin LIMIT 1";
        $result = $this->mysqli->query($query);
        
        if($result !== null || $result !== false)
        {
            $dbData = array();
            if($result->num_rows > 0) {
                while($row = $result->fetch_assoc()) {
                    $dbData[] = $row;	
                }
                // Set the public properties
                $this->Id = intval($dbData[0]['id']);
                $this->StatusText = $dbData[0]['StatusText'];
                $this->Running = intval($dbData[0]['Running']);
            
                $cronRunTime = intval($dbData[0]['LastCronRun']);
                if($cronRunTime > 0){
                    $this->LastCronRun = new DateTime("@$cronRunTime");
                }else{
                    $this->LastCronRun = "none";
                }
                
                $this->Errors = intval($dbData[0]['Errors']);
                $this->IntervaluSecs = intval($dbData[0]['IntervaluSecs']);
                $this->DomainNameServer = $dbData[0]['DomainNameServer'];
                $this->FromAddress = $dbData[0]['FromAddress'];
                $this->FromName = $dbData[0]['FromName'];
                $this->LastSent = $dbData[0]['LastSent'];            
                $this->DataValid = true;
            }
        }
    }
    
    //-----------------------------------------------------------------------
    //                         Set Properties
    //
    // Update the respective database column item on the first row only
    //-----------------------------------------------------------------------
    function SetStatusText($text)
    {
        if($this->mysqli !== null)
        {
            $query="UPDATE notificationadmin SET ".
                   "StatusText=\"".$text."\"".
                   " WHERE id=".$this->Id;
            $result = $this->mysqli->query($query);
            if($result === true)
            {
                $this->StatusText = $text;
            }
        }          
    }

    function SetRunning($value)
    {
        if($this->mysqli !== null)
        {
            $query="UPDATE notificationadmin SET ".
                   "Running=".$value.
                   " WHERE id=".$this->Id;
            $result = $this->mysqli->query($query);
            if($result === true)
            {
                $this->Running = $value;
            }
        }                          
    }    
    
    function SetLastCronRun($dateTime)
    {
        if($this->mysqli !== null)
        {
            // Convert data time to epoch
            $timeEpoch = $dateTime->getTimestamp();
            $query="UPDATE notificationadmin SET ".
                   "LastCronRun=".$timeEpoch.
                   " WHERE id=".$this->Id;
            $result = $this->mysqli->query($query);
            if($result === true)
            {
                $this->LastCronRun = $dateTime;
            }
        }                          
    }
    
    function SetLastCronRunNow()
    {       
        if($this->mysqli !== null)
        {
            $dateTime = new DateTime();;
            $query="UPDATE notificationadmin SET ".
                   "LastCronRun=".$dateTime->getTimestamp().
                   " WHERE id=".$this->Id;
            $result = $this->mysqli->query($query);
            if($result === true)
            {                
                $this->LastCronRun = $dateTime->format('Y-m-d H:i:s');
            }
        }                                  
    }
    
    function SetErrors($value)
    {
        if($this->mysqli !== null)
        {
            $query="UPDATE notificationadmin SET ".
                   "Errors=".$value.
                   " WHERE id=".$this->Id;
            $result = $this->mysqli->query($query);
            if($result === true)
            {
                $this->Errors = $value;
            }
        }                          
    }
    
    function SetIntervaluSecs($value)
    {
        if($this->mysqli !== null)
        {
            $query="UPDATE notificationadmin SET ".
                   "IntervaluSecs=".$value.
                   " WHERE id=".$this->Id;
            $result = $this->mysqli->query($query);
            if($result === true)
            {
                $this->IntervaluSecs = $value;
            }
        }                          
    }
    
    function SetDomainNameServer($value)
    {
        if($this->mysqli !== null)
        {
            $query="UPDATE notificationadmin SET ".
                   "DomainNameServer=\"".$value."\"".
                   " WHERE id=".$this->Id;
            $result = $this->mysqli->query($query);
            if($result === true)
            {
                $this->DomainNameServer = $value;
            }
        }                                  
    }
    
    function SetFromAddress($value)
    {
        if($this->mysqli !==null)
        {
            $query="UPDATE notificationadmin SET ".
                   "FromAddress=\"".$value."\"".
                   " WHERE id=".$this->Id;
            $result = $this->mysqli->query($query);
            if($result === true)
            {
                $this->FromAddress = $value;
            }            
        }
    }
    
    function SetFromName($value)
    {
        if($this->mysqli !==null)
        {
            $query="UPDATE notificationadmin SET ".
                   "FromName=\"".$value."\"".
                   " WHERE id=".$this->Id;
            $result = $this->mysqli->query($query);
            if($result === true)
            {
                $this->FromName = $value;
            }            
        }        
    }
    
    function SetLastSent($value)
    {
        if($this->mysqli !==null)
        {
            $query="UPDATE notificationadmin SET ".
                   "LastSent=".$value.
                   " WHERE id=".$this->Id;
            $result = $this->mysqli->query($query);
            if($result === true)
            {
                $this->LastSent = $value;
            }            
        }
    }
    
    function SetLastSentNow()
    {
        if($this->mysqli !==null)
        {
            $timeNow = time();
            
            $query="UPDATE notificationadmin SET ".
                   "LastSent=".$timeNow;
                   " WHERE id=".$this->Id;
            $result = $this->mysqli->query($query);
            if($result === true)
            {
                $this->LastSent = $timeNow;
            }            
        }        
    }
    
}

?>