<?php
/*******************************************************************************
* 
*    ULTRA STEREO LABS CONFIDENTIAL
*    _____________________________________________________________________
* 
*       Copyright (c) 2014 - 2015 Ultra Stereo Labs Incorporated 
*                         All Rights Reserved.
* 
*    NOTICE:  All information contained herein is, and remains
*    the property of Ultra Stereo Labs Incorporated and its suppliers,
*    if any.  The intellectual and technical concepts contained
*    herein are proprietary to Ultra Stereo Labs Incorporated
*    and its suppliers and may be covered by U.S. and Foreign Patents,
*    in process, and are protected by trade secret or copyright law.
*    Dissemination of this information or reproduction of this material
*    is strictly forbidden unless prior written permission is obtained
*    from Ultra Stereo Labs Incorporated. 181 Bonetti Drive, San Luis Obispo, 
*    California 93401-7397, USA - www.uslinc.com
*
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
*
*******************************************************************************/
//-----------------------------------------------------------------------
// classNotification
//
// This will be used to update the notifications sent database table. It
// will also include accessing other tables to determine who gets sent
// a notification(s). 
//-----------------------------------------------------------------------
class classNotification{
    //-------------------------------------------------------------------
    //                          Data
    //-------------------------------------------------------------------
    // Public properties
    public $dbConnected = false;
    
    // Private variables
    var $mysqli;
    var $emailList;
    
    //-----------------------------------------------------------------------
    // Constructor
    //
    // Created a connection with the database.
    //-----------------------------------------------------------------------
    function __construct($sqli)
    {
        $this->mysqli = $sqli;
        
        if($this->mysqli !== null)
        {
            $this->dbConnected = true;
        } 
        
        $this->emailList = array();
    }

    //-----------------------------------------------------------------------
    // GetAllUserList
    //
    // Return a list of users with a request for all or errors notifications.
    //-----------------------------------------------------------------------
    function GetAllUsersList(&$users)
    {
        global $DB_NOTIFICATIONS_NONE;

        // Retrieve the users from the database
        $query = "SELECT * FROM users";
        $result = $this->mysqli->query($query);

        if ($result->num_rows > 0) {
            while ($row = $result->fetch_assoc()) {
                // Check if user wants a notification.
                $notifyType = intval($row['NotificationType']);
                if($notifyType != $DB_NOTIFICATIONS_NONE)
                {
                   $users[] = $row;
                }
            }
        }
        return sizeof($users);               
    }
    
    //-----------------------------------------------------------------------
    // GetTheaterGroupId
    //
    // Use the theatergroups table to retrieve the Group ID.
    //  
    // The LSS is assigned an OrgKey and Theater Number that are passed into
    // this routine to perform a query the return the Group ID. If the theater
    // is not found then a -1 is returned.
    //-----------------------------------------------------------------------
    function GetTheaterGroupId($data)
    {
        $query="SELECT GroupID FROM theatergroups WHERE ".
               "TheaterNumber=".intval($data->TheaterNumber)." AND OrgKey=\"".$data->OrgKey."\" LIMIT 1";
        $result = $this->mysqli->query($query);

        if ($result->num_rows > 0) {
            $row = $result->fetch_assoc();
            $groupid = intval($row['GroupID']);
        } else {
            // Not found
            $groupid = -1;
        }
        
        return $groupid;
    }
    
    //-----------------------------------------------------------------------
    // GetGroupOrgKey 
    //
    // Use the groups table to retrieve organization.
    // 
    // For a theater group theres a an OrgKey that identifies a number of 
    // user groups. This will return the OrgKey to be used to retrieve
    // all user group members.
    //-----------------------------------------------------------------------
    function GetGroupOrgKey($grpId)
    {
        $query="SELECT PID FROM groups WHERE GID=".$grpId;
        $result = $this->mysqli->query($query);
        
        if ($result->num_rows > 0) {
            $row = $result->fetch_assoc();
            $pid = intval($row['OrgKey']);
        } else {
            // Not found
            $pid = NULL;
        }       
        return $pid;
    }
    
    //-----------------------------------------------------------------------
    // GetUserGroups($orgKey)
    //
    // Use the groups table the retrieve the list of group id(s).
    // 
    // There are groups associated by OrgKey and those are Groups of users. This
    // will return a list of user group id's.
    //-----------------------------------------------------------------------
    function GetUserGroups($orgKey)
    {
        $groupsList = array();
        
        $query="SELECT GID FROM groups WHERE OrgKey=\"".$orgKey."\"";
        $result = $this->mysqli->query($query);
        
        if ($result->num_rows > 0) {
            while ($row = $result->fetch_assoc()) {
                array_push($groupsList, intval($row['GID']));
            }
        }
        
        return $groupsList;
    }
    
    //-----------------------------------------------------------------------
    // GetUserIdList
    //
    // Use the usergroups table to retrieve a list of user id(s).
    // 
    // For each matching group id the user id is added to the list.
    //-----------------------------------------------------------------------
    function GetUserIdList($grouplist)
    {
        $userIdList = array();
        
        for($idx = 0; $idx < sizeof($grouplist); $idx++)
        {
            $query="SELECT UserID FROM usergroups WHERE GroupID=".$grouplist[$idx];
            $result = $this->mysqli->query($query);
            // Check and process the query results
            if ($result->num_rows > 0) {
                while ($row = $result->fetch_assoc()) {
                    array_push($userIdList, intval($row['UserID']));
                }
            }        
        } 
        
        return $userIdList;
    }
    
    //-----------------------------------------------------------------------
    // GetGroupList
    //
    // Creates a tree result for all subgroups up to top parent where pid = 0.
    //-----------------------------------------------------------------------
    function GetGroupList($groupId)
    {
        $groupList = array();
        
        $pid = 0;
        do{
            array_push($groupList, $groupId);
            // Get the PID for the group id
            $query="SELECT PID FROM groups WHERE GID=".$groupId;
            $result = $this->mysqli->query($query);
            // Check and process the query results
            if ($result->num_rows > 0) {
                $row = $result->fetch_assoc();
                $pid = intval($row['PID']);
            }
            else
            {
                break;
            }
            // If PID is non zero it represents a GID
            if($pid > 0)
            {
                $groupId = $pid;
            }            
        }while($pid > 0);
        return $groupList;
    }
    
    //-----------------------------------------------------------------------
    // GetUsersList
    //
    // Use the users tables to retrieve a list of users.
    // 
    // The list of user id's will be used to retrieve the respective entry
    // and update the usersList array. For each ID there's only one entry to
    // be returned.
    //-----------------------------------------------------------------------
    function GetUsersList($userIdList)
    {
        $usersList = array();
        
        for($idx = 0; $idx < sizeof($userIdList); $idx++)
        {
            $query="SELECT * FROM users WHERE UID=".$userIdList[$idx]." LIMIT 1";
            $result = $this->mysqli->query($query);
            // Check and process the query results
            if ($result->num_rows > 0) {
                while ($row = $result->fetch_assoc()) {
                    $usersList[] = $row;                    
                }
            }                   
        }
        return $usersList;
    }
    
    //-----------------------------------------------------------------------
    // GetUsersForReport
    //
    // The Raw Data in the report contains information that is used for
    // identifying the theater group. A theater can have multiple groups
    // that are service by groups of users. A user can be found within 
    // the user group.
    //-----------------------------------------------------------------------
    function GetUsersForReport($lssData)
    {
        $usersList = array();
        $groupList = array();
        
        // Theater group ID.
        $groupid = $this->GetTheaterGroupId($lssData);
        if($groupid > -1){
            // A list of group ids to retrieve a list of users
            $groupList = $this->GetGroupList($groupid);
            if(count($groupList) >0){
                // Get a list of user id's.
                $userIdList = $this->GetUserIdList($groupList);
                if(sizeof($userIdList)>0){
                    // Get the complete record for each user in the list.
                    $usersList = $this->GetUsersList($userIdList);
                }                                
            }
        }
        
        return $usersList;
    }
    
    //-----------------------------------------------------------------------
    // UpdateEmailList
    //
    // This will create a list of individual emails and then when all has 
    // beed received, then get emails will provide a filtered list of combined
    // items to be sent as one email.
    //-----------------------------------------------------------------------
    function UpdateEmailList($msgHtml, $msgText, $notificationsList)
    {
        foreach ($notificationsList as $item) {
            $this->emailList[] = $item;
        }        
    }
    
    //-----------------------------------------------------------------------
    // GetEmails
    //
    // All notification are stored in the database (notifications table).
    // This will retrieve the first instance of an un-sent email name with 
    // a SentDateTime of zero. Then for all emails for that user return a 
    // list of notification that are to be emailed.
    // 
    //   
    //-----------------------------------------------------------------------
    function GetEmails()
    {
        $userEmail = "";
        $EmailArray = Array();
        
        // Get first record for an unsent email
        $query = "SELECT * FROM notifications WHERE SentDateTime=0 LIMIT 1";
        $result = $this->mysqli->query($query);
        if ($result->num_rows > 0) 
        {
            $row = $result->fetch_assoc();
            $userEmail = $row['Email'];
            
            // Use the email name to retrive all pending emails for that user.
            $query = "SELECT * FROM notifications WHERE SentDateTime=0 AND ".
                     "Email=\"".$userEmail."\"";
            $result = $this->mysqli->query($query);
            if ($result->num_rows > 0) 
            {
                while ($row = $result->fetch_assoc()) {
                    $EmailArray[] = $row; 
                }
            }            
        }        
        return $EmailArray;
    }
    
    //-----------------------------------------------------------------------
    // GetUserInfo
    //
    // Use the email name to retrieve the user name.
    //-----------------------------------------------------------------------
    function GetUserInfo($email)
    {
        $userInfo = array();
        $query = "SELECT UserName, NotificationType FROM users WHERE Email=\"".$email."\"LIMIT 1";
        $result = $this->mysqli->query($query);
        if ($result->num_rows > 0)
        {
            $row = $result->fetch_assoc();
            $userInfo = $row;
        }
        return $userInfo;
    }
    
    //-----------------------------------------------------------------------
    // GetRawDataRecord
    //
    // For each of the notifications there's a RawData reference that is 
    // retrieved for composing an email.
    //-----------------------------------------------------------------------
    function GetRawDataRecord($rawId)
    {
        $rawData=null;
        
        $query = "SELECT * from RawData WHERE id=$rawId LIMIT 1";
        $result = $this->mysqli->query($query);
        if ($result->num_rows > 0) 
        {
            $row = $result->fetch_assoc();
            $rawData = $row;
        }
        return $rawData;
    }
    
    //-----------------------------------------------------------------------
    // UpdateNotificationStatus
    //
    // The RawDataId itendtifies the record to be updated in the email
    // being sent. 
    //-----------------------------------------------------------------------
    function UpdateNotificationStatus($rawId, $email, $Status)
    {
        $query="UPDATE notifications SET SentDateTime=".$Status." ".
               "WHERE RawDataId=".$rawId." AND Email=\"".$email."\"";
        $this->mysqli->query($query);
    }
    
    //-----------------------------------------------------------------------
    // UpdateNotificationFailedStatus
    //
    // Update the entries with matching raw data id to failed. This is the
    // case if the raw data has been deleted prior to executing the processing
    // of the pending email notifications.
    //-----------------------------------------------------------------------
    function UpdateNotificationFailedStatus($rawId, $Status)
    {
        $query="UPDATE notifications SET SentDateTime=".$Status." ".
               "WHERE RawDataId=".$rawId;
        $this->mysqli->query($query);
        
    }
    
    //-----------------------------------------------------------------------
    // SaveNotification
    //
    // When a notification is sent, this is called to update the 
    // database.
    // 
    // The item is an accociative array that requires the defined keys names.
    // The data source is from the post.php the lss web service for posted
    // results. Note that the numeric values are strings.
    //-----------------------------------------------------------------------
    function SaveNotification($data)
    {
        if(count($data)>0)
        {
            foreach ($data as $item) {
                $query="INSERT INTO notifications ".
                    "(RawDataId,ReceivedDateTime,Email,SentDateTime)".
                    " VALUES ".
                    "(".$item['RawDataId'].",".
                    $item['ReceivedDateTime'].",".
                    "\"".$item['Email']."\",".
                    $item['SentDateTime'].")";
                $this->mysqli->query($query);     
            }
        }
    }
    
}

?>
