//---------------------------------------------------------------------------
// dbRequestService
// 
// The service provides a low level client interface that in this case is 
// used to access the server side service via a http post method. The request
// and parameters are sent to the server side php via a string conversion of
// a json object. The php server side function will then decode the data and 
// execute the respective command action.
//---------------------------------------------------------------------------
app.service('dbRequestService', function ($http, Data) {

    // A generic server side php service request.
    this.dbRequest = function (dataList, cmdToken, value1, value2) {
        console.log("dbRequestService: post command = "+cmdToken); 
        // The command and parameters are passed the service via 
        // a post.
        var jsonCmdObj = {"command":cmdToken,
                          "param1":value1,
                          "param2":value2}; 
        
        var cmdData = JSON.stringify(jsonCmdObj);
        //console.log("AJAX server request "+JSON.stringify(jsonCmdObj)); //For Debugging
        $http.post('serverservices/requestData.php', cmdData).success(function(data){
            if(data.status === 'error')
                Data.toast(data);
            else
            {
                // update the data list that is being observed by the view
                // such as a table. The fields are as defined in the data base
                // column definitions.
                dataList.length=0;
                var item = {};
                for (var i in data)
                {   
                    switch(cmdToken)
                    {
                        case EnumDbCmds.ALL:
                            item = new AllStruct(data[i].TheaterName, 
                                                 data[i].TheaterNumber,
                                                 data[i].AuditoriumNumber,
                                                 data[i].Description,
                                                 data[i].Value,
                                                 data[i].Units, 
                                                 data[i].Minimum, 
                                                 data[i].Maximum,
                                                 data[i].SerialNumber);
                            break;
                        case EnumDbCmds.THEATER_NAME:
                            item = new AllStruct(data[i].TheaterName, 
                                                 data[i].TheaterNumber,
                                                 data[i].AuditoriumNumber,
                                                 data[i].Description,
                                                 data[i].Value,
                                                 data[i].Units, 
                                                 data[i].Minimum, 
                                                 data[i].Maximum,
                                                 data[i].SerialNumber);
                            break;                            
                        case EnumDbCmds.OVERVIEW:
                            item = new OverviewStruct(data[i].TheaterName,
                                                      data[i].TheaterNumber, 
                                                      data[i].AuditoriumNumber,
                                                      data[i].OrgKey);
                            break;
                        case EnumDbCmds.USERS:
                            item = new UsersStruct(data[i].UID,
                                                   data[i].UserName,
                                                   data[i].Company,
                                                   parseInt(data[i].NotificationType),
                                                   parseInt(data[i].Level),
                                                   data[i].Email,
                                                   data[i].Phone,
                                                   data[i].Created);
                            break;
                        case EnumDbCmds.GROUPS:
                            item = new GroupsStruct(data[i].GID,
                                                    data[i].PID,
                                                    data[i].GroupName,
                                                    data[i].OrgKey);
                            break;
                        case EnumDbCmds.THEATERGROUPS:
                            item = new TheaterGroupsStruct(data[i].TGID,
                                                           data[i].TheaterNumber,
                                                           data[i].OrgKey,
                                                           data[i].GroupID);
                            break;
                        case EnumDbCmds.USERGROUPS:
                            item = new UserGroupsStruct(data[i].UGID,
                                                        data[i].UserID,
                                                        data[i].GroupID,
                                                        data[i].Active);
                            break;
                        case EnumDbCmds.THEATER_OVERVIEW:                
                            item = new TheaterOverviewStruct(data[i].TheaterName,
                                                             data[i].TheaterNumber, 
                                                             data[i].AuditoriumNumber,
                                                             data[i].OutsideLimits,
                                                             data[i].Description,
                                                             data[i].Value,
                                                             data[i].DateTime,
                                                             data[i].Minimum,
                                                             data[i].Maximum,
                                                             data[i].Units,
                                                             data[i].OrgKey);
                            break;
                        case EnumDbCmds.LSS_DATA:                
                            item = new LSSDataStruct(data[i].Hardware,
                                                             data[i].Firmware, 
                                                             data[i].IpAddress,
                                                             data[i].SerialNumber,
                                                             data[i].Comments);
                                                             
                            break;
                        case EnumDbCmds.COLLECT_THEATER_GROUPS:                
                            item = new TheaterGroupCollectionStruct(
                                                        data[i].TheaterNumber,
                                                        data[i].OrgKey,
                                                        data[i].TheaterName); 
                           break;

                        case EnumDbCmds.ORGKEYS:                
                            item = new OrgKeyCollectionStruct(i, data[i].Company);
                            break;
                        case EnumDbCmds.NOTIFICATIONS:
                           item = new NotificationDataStruct(
                                                        data[i].SentDateTime,
                                                        data[i].ReceivedDateTime,
                                                        data[i].RawDataId); 
                           break;
                        case EnumDbCmds.MY_GROUPS:
                            item = new GroupsStruct(data[i].GID,
                                                    data[i].PID,
                                                    data[i].GroupName,
                                                    data[i].OrgKey);
                            break;
                        case EnumDbCmds.MY_THEATERGROUPS:
                            item = new TheaterGroupsStruct("",
                                                           data[i].TheaterNumber,
                                                           data[i].OrgKey,
                                                           data[i].GroupID);
                            break;
                    }
                    if(item !== null){dataList.push(item);}
                }
            }
        }).error(function(data){
            console.log('dbRequestService: Error '+data);
        });
    };
    // An object based graph request
    // Takes an object instead of 2 strings for parameters
    this.dbObjRequest = function (dataList, cmdToken, obj) {
        console.log("dbRequestService: post command = "+cmdToken); 
        // The command and parameters are passed the service via 
        // a post.
        var jsonCmdObj = {"command":cmdToken}
        jQuery.extend(jsonCmdObj, obj);
        var cmdData = JSON.stringify(jsonCmdObj);
        //console.log("AJAX server request "+JSON.stringify(jsonCmdObj)); //For Debugging
        $http.post('serverservices/requestData.php', cmdData).success(function(data){
            if(data.status === 'error')
                Data.toast(data);
            else
            {
                // update the data list that is being observed by the view
                // such as a table. The fields are as defined in the data base
                // column definitions.
                dataList.length=0;
                var item = {};
                for (var i in data)
                {   
                    switch(cmdToken)
                    {
                        case EnumDbCmds.LSS_DATA:                
                            item = new LSSDataStruct(data[i].Hardware,
                                                             data[i].Firmware, 
                                                             data[i].IpAddress,
                                                             data[i].SerialNumber,
                                                             data[i].Comments);                           
                            break;
                    }
                    if(item !== null){dataList.push(item);}
                }
            }
        }).error(function(data){
            console.log('dbRequestService: Error '+data);
        });
    };
    // To simplify graphing, build a correctly formatted data return with a second function.
    // Alternative would be to use main return and then build graphing data array in the controller.
    this.dbGraphRequest = function (dataList, cmdToken, StartDate, EndDate, TheaterNumber, AuditoriumNumber, Description, OrgKey) {
        console.log("dbGraphRequestService: post command = "+cmdToken); 
        // The command and parameters are passed the service via 
        // a post.
        var jsonCmdObj = {"command":cmdToken,
                          "StartDate":StartDate,
                          "EndDate":EndDate,
                          "TheaterNumber":TheaterNumber,
                          "AuditoriumNumber":AuditoriumNumber,
                          "Description":Description,
                          "OrgKey":OrgKey};
        var cmdData = JSON.stringify(jsonCmdObj);
        $http.post('serverservices/requestData.php', cmdData).success(function(data){
            if(data.status === 'error'){
                          Data.toast(data);
            }
            else
            {
                // update the data list that is being observed by the view
                // such as a table.
                // For Graphing there are 3 arrays, Value, Max and Min.
                dataList[0].values.length=0;
                dataList[1].values.length=0;
                dataList[2].values.length=0;
                var item = {};
                for (var i in data)
                {             
                    item = new GraphDataStruct(data[i].datetime,data[i].value, data[i].units);
                    itemMax = new GraphDataStruct(data[i].datetime,data[i].Maximum, "");
                    itemMin = new GraphDataStruct(data[i].datetime,data[i].Minimum, "");
                    if(item !== null){
                        dataList[0].values.push(item);
                        if(itemMax !== null && data[i].Maximum < 90000000){ //90000000000 is the NULL placeholder reported from the LSS
                            dataList[1].values.push(itemMax);
                        }
                        if(itemMin !== null && data[i].Minimum > -9000000){
                            dataList[2].values.push(itemMin);
                        }
                    }
                }
                if(dataList[0].values.length !== 0){
                    if(dataList[1].values.length === 0){dataList[1].values.push({x:dataList[0].values[0].x,y:null,units:''});}
                    if(dataList[2].values.length === 0){dataList[2].values.push({x:dataList[0].values[0].x,y:null,units:''});}
                }
                console.log("Graph data returned "+dataList[0].values.length);
            }
        }).error(function(data){
            console.log('GraphError: '+data);
        });
    };
    this.dbDualGraphRequest = function (dataList, cmdToken, StartDate, EndDate, TheaterNumber, AuditoriumNumber, Description) {
        console.log("dbGraphRequestService: post command = "+cmdToken); 
        // The command and parameters are passed the service via 
        // a post.
        var jsonCmdObj = {"command":cmdToken,
                          "StartDate":StartDate,
                          "EndDate":EndDate,
                          "TheaterNumber":TheaterNumber,
                          "AuditoriumNumber":AuditoriumNumber,
                          "Description":Description};
        var cmdData = JSON.stringify(jsonCmdObj);
        alert(cmdData);
        $http.post('serverservices/requestData.php', cmdData).success(function(data){
            if(data.status === 'error'){
                          Data.toast(data);
            }
            else
            {
                // update the data list that is being observed by the view
                // such as a table.
                // For Graphing there are 3 arrays, Value, Max and Min.
                var i = 0;
                var item = {};
                alert(JSON.stringify(dataList));
                alert(JSON.stringify(data));
                for (i in data){             
                    item = $.grep(dataList[0].values, function(e){ return e.x === parseFloat(data[i].datetime); });
                    if( item.length > 0 ){
                        item[0].x = data[i].value;
                        item[0].unitsX = data[i].units;
                    }
                    alert(JSON.stringify(item));
                }
                i=0;
                while( i <= dataList[0].values.length){
                    if(dataList[0].values[i].unitsX===null || dataList[0].values[i].unitsX===undefined){
                        dataList[0].values.splice(i,1);
                    }
                    else { i=i+1;}
                }
                dataList.length=1;
                alert(JSON.stringify(dataList));
                console.log("Graph data returned "+dataList[0].values.length);
            }
        }).error(function(data){
            console.log('GraphError: '+data);
        });
    };   
  
    this.dbRawDataRequest = function (dataList, cmdToken, paramsObj) {
        console.log("dbRawDataRequest Sent"); 
        // The command and parameters are passed the service via 
        // a post.
        var cmdData = JSON.stringify({"command":cmdToken,"data":paramsObj});
        $http.post('serverservices/requestData.php', cmdData).success(function(data){
            if(data.status === 'error'){
                          Data.toast(data);
            }
            else
            {
                // update the data list that is being observed by the view
                // such as a table. The fields are as defined in the data base
                // column definitions.
                dataList.length=0;
                var item = {};
                //console.log(JSON.stringify(data)); //for debugging
                for (var i in data)
                {             
                    item = new TheaterOverviewStruct(data[i].TheaterName,
                                                    data[i].TheaterNumber, 
                                                    data[i].AuditoriumNumber,
                                                    data[i].OutsideLimits,
                                                    data[i].Description,
                                                    data[i].Value,
                                                    data[i].DateTime,
                                                    data[i].Minimum,
                                                    data[i].Maximum,
                                                    data[i].Units,
                                                    data[i].OrgKey);
                    if(item !== null){ dataList.push(item); }
                }
            }
        }).error(function(data){
            console.log('Error: '+data);
            Data.toast(data);
        });
    };

});

